/*-------------------------------------------------------------------------*

             SWIS -- THE STARBUCK WINDOWED INFORMATION SYSTEM(c)
                        Copyright 1990, 1991, 1992
                  by Starbuck & Staff, Tarzana, California
                SWIS(tm) is the trademark of Starbuck & Staff

                                  NOTICE:

    Place in the public domain by the author on 03/18/92.  No warranties
    of any kind expressed or implied.  Freeware.


*--------------------------------------------------------------------------*

    Title.........:  PUSH-BUTTONS WITH POP-OUT ACTION

    Description...:  Keyboard activated push-buttons which respond to
                     cursor movement, enter-key and trigger key input.
                     When a button is selected, the selection is 
                     displayed on screen with a push-down, pop-up action.

    Author........:  Wendy Starbuck


*--------------------------------------------------------------------------*/


// Include standard headers
#include "box.ch"
#include "inkey.ch"

// Define constants
#define  K_SPACE  32

    local cSaveColor, nAction := 0, nStart := 1

    clear screen
    set scoreboard off

    cSaveColor := SetColor( "W+/R" )
    @ 09, 20, 15, 59 box B_SINGLE + space(1)
    @ 11, 25 say "Do you want to push a button?"
    while nAction != 3 .and. Lastkey() != K_ESC
        nAction := Button_Horizon( 12, 22, @nStart, ;
                                   { "  YES  ", " MAYBE ", "  NO   "  } )
    end

    SetColor( cSaveColor )
    set cursor on
    clear screen

quit


/*-------------------------------------------------------------------------*
    Function....:  Button_Horizon
    Description.:  Display push buttons horizontally.
*--------------------------------------------------------------------------*/

function Button_Horizon ( nRow, nCol, nSelect, aButton )

    local cColor1, cColor2, cColor3
    local x, nOffSet, cSelect, lExit, nPop, cTrigger := ""

    // Setup the button colors
    cSavColor := setcolor()
    cColor1 := "N/" + Substr( cSavColor, At( "/", cSavColor )+1 )
    cColor2 := "W+/G"
    cColor3 := "N/W"

    // Define the trigger keys
    x := 0
    nNum := len( aButton )
    while nNum > x++
        cTrigger := cTrigger + substr( alltrim( aButton[x] ), 1, 1 )
    end

    // Define the controls
    set cursor off
    nOffSet := 0
    cSelect := ""
    lExit := .F.
    x := 1

    while .T.

        // Display buttons until end of array
        SetColor( cColor1 )
        if nNum >= x

            // Graphic Style Button
            if nSelect == x
                Button_Push( nRow, nCol, nOffSet, aButton[x], cColor2, lExit )
                nPop := nOffSet
            else
                Button_Push( nRow, nCol, nOffSet, aButton[x], cColor3 )
            endif
            nOffSet := ( Len( aButton[x] ) + 4 ) * x
            x++

        else

            // If a button has been selected, exit
            if lExit
                // Handle pop-out action
                Inkey(.3)
                Button_Push( nRow, nCol, nPop, aButton[nSelect], cColor2 )
                Inkey(.1)
                exit
            else
                // Wait for a button press
                cSelect := Chr( Inkey(0) )
            endif

            do case
            case LastKey() == K_ENTER .or. ;
                 LastKey() == K_SPACE
                lExit := .T.
            case Upper( cSelect ) $ cTrigger // Select trigger key
                lExit := .T.
                nSelect := At( Upper( cSelect ), cTrigger )
            case LastKey() == K_LEFT         // Left arrow pressed
                nSelect := if( nSelect - 1 < 1, nNum, nSelect - 1 )
            case LastKey() == K_RIGHT        // Right arrow pressed
                nSelect := if( nSelect + 1 > nNum, 1, nSelect + 1 )
            case LastKey() == K_ESC          // Escape out
                nSelect := 0
                exit
            endcase
            nOffSet := 0
            x := 1

        endif

    end

    // Housekeeping
    SetColor( cSavColor )

return nSelect


/*-------------------------------------------------------------------------*
    Function....:  Button_Push 
    Description.:  Button - push button style.
*--------------------------------------------------------------------------*/

function Button_Push ( nRow, nCol, nOffSet, cName, cColor, lExit )

    local nLen := Len( cName )

    cColor := if( cColor == NIL, SetColor(), cColor )
    lExit := if( lExit == NIL, .F., lExit )
    nCol := nCol + nOffSet

    if lExit
        // push state
        @ nRow+1, nCol+2 say space( nLen + 3 )
        @ nRow+2, nCol+2 say space( nLen + 3 )
        SetColor( cColor )
        @ nRow+1, nCol+3 say " " + cName + " "
    else
        // normal state
        @ nRow+1, nCol+nLen+4 say ""
        @ nRow+2, nCol+3 say Replicate( "", nLen + 2 )
        SetColor( cColor )
        @ nRow+1, nCol+2 say " " + cName + " "
    endif

return (.T.)


/* EOF: PUSHBU.PRG -----------------------------------------------------*/
