/*
 * Radio buttons for Clipper 5.01.
 *
 * For a demonstration, run the batch file MAKEDEMO.BAT or make an EXE using
 * the command:
 *
 *           rmake radiobtn /dTEST
 *
 * I'd like to hear about any enhancements you make to these functions.
 * My CompuServe account is [70451,2312].  My mailing address is:
 *
 *           Dan Comeau
 *           P.O. Box 6903, Station J
 *           Ottawa, Ontario, K2A 3Z5, Canada.
 *
 * Ŀ
 *  The functions contained herein are the original work of Dan Comeau 
 *  and are placed in the public domain.                               
 * 
*/

#include "radiobtn.ch"

#include "box.ch"          // as shipped with Clipper 5.01
#include "getexit.ch"      // ''   ''     ''     ''    ''
#include "inkey.ch"        // ''   ''     ''     ''    ''

#define HORIZSPACING 6     // space (ie.,"  ( ) ") between text of horizontal choices
#define K_SPACEBAR 32

static aAllButtons := {}   // for all the get radio buttons


/*
* 
*  Test function for the radio buttons.
*
*  To make the test version, use:  rmake radiobtn /dTEST
* 
*/
#ifdef TEST

  #define SCREEN_COLOR "GR+/B"
  #define GREYED_COLOR "W/B"

  function test()

    local cGet1    := "Get 1"
    local cGet2    := "Get 2"
    local GetList  := {}
    local nChoice1 := 2          // initial choices for radio buttons
    local nChoice2 := 1
    local nChoice3 := 1
    local nChoice4 := 3
    local nChoice5 := 2

    set scoreboard off

    // demonstrate set key ability within a radio button GET
    setkey( K_F1, { |s,r,c| s:=savescreen(), ;
                            r:=row(),;
                            c:=col(),;
                            scroll(), ;
                            setpos(5,5), ;
                            dispout("You pressed F1.  Press any key to continue . . ."), ;
                            inkey(0), ;
                            restscreen(,,,,s),;
                            setpos(r,c) } )

    setcolor( SCREEN_COLOR )
    clear screen

    @ 2,5  say "Use the Tab, Shift-Tab, arrow, space bar, and Enter keys to move around."
    @ 3,5  say "Press F1 to show use of setkey within GETs."
    @ 5,5  say "Normal Get #1:" ;
           get cGet1
    @ 6,5  say "Buttons #1" ;
           get nChoice1 ;
           color SCREEN_COLOR ;
           with radiobuttons { "1 Choice1","2 Disable Buttons #2","3 Choice1" }
    @ 6,40 say "Buttons #2" ;
           get nChoice2 ;
           when nChoice1 != 2 ;
           color SCREEN_COLOR+","+GREYED_COLOR ;
           with radiobuttons { "1 Choice2","2 Choice2","3 Choice2","4 Choice2" } ;
           nobox
    @ 7,60 get nChoice3 ;
           color SCREEN_COLOR ;
           with radiobuttons { "1 Choice3","2 Choice3","3 Choice3","4 Choice3","5 Choice3" } ;
           nobox
    @ 11,5 say "Normal Get #2:" ;
           get cGet2
    @ 13,5 say "Buttons #4:"
    @ 13,col()+1 get nChoice4 ;
                 color SCREEN_COLOR ;
                 with radiobuttons { "1 Choice4","2 Choice4","3 Choice4" } ;
                 nobox ;
                 horizontal
    @ 16,5 get nChoice5 ;
           color SCREEN_COLOR ;
           with radiobuttons { "1 Choice5","2 Choice5","3 Choice5" } ;
           double ;
           horizontal
    read
    RadioBtnKill()  // reduce memory requirement by setting array to NIL

    // display values
    ? "Normal Get Values  :",cGet1, cGet2
    ? "Radio Button Values:", nChoice1, nChoice2, nChoice3, nChoice4, nChoice5

  return nil

#endif


/*
* 
*  Initialization for Radio Buttons.  Display title and choices.  Optionally
*  draw box around choices.  Horizontal choices must fit on one line.
* 
*/
function RadioBtnNew( oGet, bWhen,                      ;
                      nRow, nCol, cTitle, nChoice,      ;
                      aChoices, lNoBox, lDblBox, lHoriz )

  local cColorSpec    // color string
  local n             // temp variable
  local nWidth        // width of button box

  if cTitle == NIL
    cTitle := ""      // init to enable testing in len()
  endif

  if nChoice < 1 .or. nChoice > len( aChoices )  // make sure nChoice is in valid range
    nChoice := 1
  endif

  // Add choices array to the aAllButtons array.
  aadd( aAllButtons, { oGet:Name, aChoices } )

  // Ŀ
  //  Draw box around buttons 
  // 
  if ! lNoBox   // draw box around buttons
    if lHoriz   // draw horizontal box

      // find total width of aChoices choices
      nWidth := 0
      aeval( aChoices, { |c| nWidth += len(c)+HORIZSPACING } )
      nWidth := max( nWidth, len(cTitle)+1 )

      // draw single or double line box
      dispbox( nRow, nCol, nRow+2, nCol+nWidth+1, ;
               if( lDblBox, B_DOUBLE, B_SINGLE )+space(1), oGet:ColorSpec )

    else        // draw vertical box

      // find max width of aChoices choices
      nWidth := len( aChoices[1] )
      aeval( aChoices, { |c| nWidth := max(nWidth, len(c)) } )
      *-- nWidth += 5     // add space for " ( ) "
      nWidth := max( nWidth+5, len(cTitle)+1 )    // add 5 spaces for " ( ) "

      // draw single or double line box
      dispbox( nRow, nCol, nRow+len(aChoices)+1, nCol+nWidth+2, ;
               if( lDblBox, B_DOUBLE, B_SINGLE )+space(1), oGet:ColorSpec )

    endif
  endif

  // Ŀ
  //  Put title at top left corner 
  // 
  if !empty( cTitle )
    if lNoBox   // no box around buttons
      @ nRow, nCol say cTitle color oGet:ColorSpec
    else        // draw box around buttons
      @ nRow, nCol+1 say " "+cTitle+" " color oGet:ColorSpec
    endif
  endif

  // Ŀ
  //  Display radio button choices 
  // 
  // check when condition for this get; use this to set colors
  if ( bWhen == NIL ) .or. eval( bWhen, oGet )
    // normal color
    cColorSpec := oGet:ColorSpec
  else
    // failed pre-validation (ie., WHEN)
    // grey out the radio button box choices
    cColorSpec := if( (n:=at(",",oGet:ColorSpec)) > 0, ;  // find comma delimiter
                  substr(oGet:ColorSpec,n+1), ;           // remainder of color string
                  oGet:ColorSpec )                        // same color as regular
    nChoice := 0   // don't show any choices for greyed out radio buttons
  endif

  // draw the buttons
  /*
  *  Note: 2 is subtracted from nCol when horizontal and no box or
  *        title. This was needed to line up the buttons with the oGet
  *        supplied coordinates when DrawRadioButtons() is called from
  *        RadioBtnReader().
  */
  DrawRadioButtons( nRow, ;
                    nCol-if(lHoriz .and. lNoBox .and. empty(cTitle), 2, 0), ;
                    aChoices, nChoice, nChoice, cColorSpec, ;
                    lNoBox, lHoriz, empty(cTitle) )

return nil


/*
* 
*  Draw Radio Buttons choices
* 
*/
static function DrawRadioButtons( nRow, nCol, aChoices, nChoice, nCursor, ;
                                  cColorSpec, lNoBox, lHoriz, lNoTitle )

  local nCursorPos := 0   // cursor position (could be either row or col)

  dispbegin()
  set cursor off

  if lHoriz   // horizontal radio buttons
    if lNoBox
      setpos( nRow+if(lNoTitle,0,1), nCol )
      aeval( aChoices, { |c,n| setpos(row(), col()), ;
                               dispout("("+ ( if( n==nCursor, nCursorPos:=col(), ),;
                                              if( n==nChoice, "", " " );
                                            ) +;
                                       ") "+c+" ", ;
                                       cColorSpec) } )
      setpos( nRow+if(lNoTitle,0,1), nCursorPos+1 )  // display cursor

    else   // with a box around buttons
      setpos( nRow+1, nCol+1 )
      aeval( aChoices, { |c,n| setpos(row(), col()), ;
                               dispout(" ("+ ( if( n==nCursor, nCursorPos:=col(), ),;
                                               if( n==nChoice, "", " " );
                                             ) +;
                                       ") "+c+" ", ;
                                       cColorSpec) } )
      setpos( nRow+1, nCursorPos+2 )  // display cursor
    endif

  else        // vertical radio buttons
    if lNoBox
      setpos( nRow-if(lNoTitle,1,0), nCol )
      aeval( aChoices, { |c,n| setpos(row()+1, nCol), ;
                               dispout("("+ ( if( n==nCursor, nCursorPos:=row(), ),;
                                              if( n==nChoice, "", " " );
                                            ) +;
                                       ") "+c, ;
                                       cColorSpec) } )
      setpos( nCursorPos, nCol+1 )  // display cursor
    else
      setpos( nRow, nCol )
      aeval( aChoices, { |c,n| setpos(row()+1, nCol+2), ;
                               dispout("("+ ( if( n==nCursor, nCursorPos:=row(), ),;
                                              if( n==nChoice, "", " " );
                                            ) +;
                                       ") "+c, ;
                                       cColorSpec) } )
      setpos( nCursorPos, nCol+3 )  // display cursor
    endif
  endif

  set cursor on
  dispend()

return nil


/*
* 
*  Radio Buttons GET Reader.
*  Supports WHEN but not VALID.
* 
*/
function RadioBtnReader( oGet, lNoBox, lHoriz, cTitle )

  local aChoices      // radio button choices
  local cGetVar       // current get variable
  local cGreyColor    // greyed out color if WHEN condition failed
  local cSavedScreen  // to save portion of screen normally showing GET value
  local nChoice       // button choices (1st one is name of get variable)
  local nCursor       // button cursor (may be different than nChoice)
  local nKey          // key pressed
  local nMaxChoices   // max number of choices
  local nOldChoice    // to save current choice
  local nOldCursor    // to save current cursor position
  local n             // temp variable
  local bHotKey       // code block for a set key is that is pressed

  // initialize variables
  aChoices := aAllButtons[ ascan( aAllButtons, { |a| a[1] == oGet:Name } ), 2 ]

  // read the GET if the WHEN condition is satisfied
  if ( GetPreValidate( oGet ) )  // note: see our own version of this udf below

    // initialize variables
    n           := 0
    nChoice     := oGet:VarGet()  // make a copy of the get var value
    nCursor     := nChoice        // cursor position
    nKey        := 0
    nMaxChoices := len( aChoices )

    // activate the GET for reading
    dispbegin()
    // save the 1 character spot where the GET value is about to be displayed
    cSavedScreen := savescreen( oGet:row, oGet:col, oGet:row, oGet:col )
    oGet:SetFocus()
    // restore the 1 character spot where the GET displayed its value
    restscreen( oGet:row, oGet:col, oGet:row, oGet:col, cSavedScreen )
    // redraw buttons: sets cursor under choice
    DrawRadioButtons( oGet:Row, oGet:Col, aChoices, nChoice, nCursor, ;
                      oGet:ColorSpec, lNoBox, lHoriz, empty(cTitle) )
    dispend()

    do while ( oGet:ExitState == GE_NOEXIT )

      nOldChoice := nChoice      // save "old" choice before movement
      nOldCursor := nCursor      // save "old" cursor choice before movement
      nKey       := inkey(0)     // wait for a key to be pressed

      // see if a hot key was pressed

      if ( bHotKey := setkey( nKey ) ) != nil
        eval( bHotKey, procname(1), procline(1), readvar() )
        loop  // get next key
      endif

      // determine what key was pressed

      do case
      case nKey == K_ESC        // cancel selection
        oGet:ExitState := GE_ESCAPE

      case nKey == K_SPACEBAR   // move to cursor or the next radio button choice
        if ! nCursor == nChoice
          // move choice to cursor position
          nChoice := nCursor
        else
          // move choice to next button
          nCursor := nChoice := if( nChoice == nMaxChoices, 1, nChoice+1 )
        endif
        
      case nKey == K_ENTER      // get to the next get
        oGet:ExitState := GE_ENTER

      case nKey == K_UP         // up arrow
        if lHoriz               // horizontal box: exit to previous get
          oGet:exitstate := GE_UP
        else                    // vertical box: move cursor up
          if nCursor == 1
            oGET:exitstate := GE_UP  // move to previous get
          else
            nCursor--
          endif
        endif

      case nKey == K_DOWN       // down arrow
        if lHoriz               // horizontal box: exit to next get
          oGET:exitstate := GE_DOWN
        else                    // vertical box: move cursor down
          if nCursor == nMaxChoices
            oGET:exitstate := GE_DOWN  // move to next get
          else
            nCursor++
          endif
        endif

      case nKey == K_LEFT       // left arrow
        if lHoriz               // horizontal box: move cursor to previous choice
          if nCursor == 1
            nCursor := nMaxChoices
            // to move to the previous get,
            // comment the line above and uncomment the next line
            // oGET:exitstate := GE_UP  // move to previous get
          else
            nCursor--
          endif
        else                    // vertical box
          // uncomment this line if you want the cursor to move to previous get
          // oGet:exitstate := GE_UP
        endif

      case nKey == K_RIGHT      // right arrow
        if lHoriz               // horizontal box: move cursor to next choice
          if nCursor == nMaxChoices
            nCursor := 1
            // to move to the next get,
            // comment the line above and uncomment the next line
            // oGET:exitstate := GE_DOWN  // move to next get
          else
            nCursor++
          endif
        else                    // vertical box
          // uncomment this line if you want the cursor to move to next get
          // oGET:exitstate := GE_DOWN
        endif

      case nKey == K_TAB        // tab: exit to next get
        oGET:exitstate := GE_DOWN

      case nKey == K_SH_TAB     // shift-tab: exit to previous get
        oGet:exitstate := GE_UP

      case nKey == K_PGUP       // page up
        oGET:ExitState := GE_WRITE

      case nKey == K_PGDN       // page down
        oGet:ExitState := GE_WRITE

      otherwise
        // handle if user pressed a key to select the first letter
        // 1st, continue search from current location
        n := ascan( aChoices, ;
                    { |c| upper( left(c,1) ) == upper ( chr(nKey) ) },;
                    nChoice+1, nMaxChoices )
        if n == 0
          // 2nd, if another not found, restart search from the top
          n := ascan( aChoices, ;
                      { |c| upper( left(c,1) ) == upper ( chr(nKey) ) },;
                      1, nChoice - 1 )
        endif
        nCursor := nChoice := if( n > 0, n, nChoice )  // move cursor if a match

      endcase

      // check if moved to new radio button selection
      if ! nOldChoice == nChoice .or. ! nOldCursor == nCursor
        DrawRadioButtons( oGet:Row, oGet:Col, aChoices, nChoice, nCursor, ;
                          oGet:ColorSpec, lNoBox, lHoriz, empty(cTitle) )
      endif

    enddo

    oGet:VarPut( nChoice )   // update get var

    // de-activate the GET
    dispbegin()
    // save the 1 character spot where the GET value is about to be displayed
    cSavedScreen := savescreen( oGet:row, oGet:col, oGet:row, oGet:col )
    oGet:KillFocus()
    // restore the 1 character spot where the GET displayed its value
    restscreen( oGet:row, oGet:col, oGet:row, oGet:col, cSavedScreen )
    dispend()

  else
    // failed pre-validation (ie., WHEN)
    // grey out the radio button box choices
    cGreyColor := if( (n:=at(",",oGet:ColorSpec)) > 0, ;  // find comma dilimiter
                  substr(oGet:ColorSpec,n+1), ;           // remainder of color string
                  oGet:ColorSpec )                        // same color as regular
    DrawRadioButtons( oGet:Row, oGet:Col, aChoices, nChoice, nCursor, ;
                      cGreyColor, lNoBox, lHoriz, empty(cTitle) )

  endif

return nil


//
// A copy from Nantucket's version with some modifications.
//
/***
*	GetPreValidate()
*	Test entry condition (WHEN clause) for a GET.
*/
static function GetPreValidate(get)

local saveUpdated
local when := .t.

	if ( get:preBlock <> NIL )

    // saveUpdated := Updated

		when := Eval(get:preBlock, get)

    // get:Display()

    // ShowScoreBoard()
    // Updated := saveUpdated

	end

  // if ( KillRead )
  //   when := .f.
  //   get:exitState := GE_ESCAPE    // provokes ReadModal() exit

  // elseif ( !when )
  if ( !when )
    get:exitState := GE_WHEN    // indicates failure

	else
		get:exitState := GE_NOEXIT		// prepares for editing

	end

return (when)


/*
* 
*  Clear the radio button array.  Do this after the READ to free up memory.
* 
*/
function RadioBtnKill()
  aAllButtons := {}
return nil

*: EOF: RADIOBTN.PRG
